" hilinks.vim: the source herein generates a trace of what
"             highlighting groups link to what highlighting groups
"
"  Last Change: August 1, 2001
"  Maintainer:  Charles E. Campbell, Jr. <charles.e.campbell.1@gsfc.nasa.gov>
"  NOTE:        This script requires Vim 6.0k (or later)
"               The script uses the @a register
"
"  Usage:
"
"  \hlt   : will reveal a linked list of highlighting from the top-level down
"           to the bottom level.
"           You may redefine the leading character using "let mapleader= ..."
"           in your <.vimrc>.
if exists("loaded_hilink") || &cp
  finish
endif
let loaded_hilink= 1

" Public Interface
if !hasmapto('<Plug>HiLinkTrace')
  map <unique> <Leader>hlt <Plug>HiLinkTrace
endif

" Global Maps
map <silent> <unique> <script> <Plug>HiLinkTrace :call <SID>HiLinkTrace()<CR>
"
" ---------------------------------------------------------------------

" HiLinkTrace: this function traces the highlighting group names
"             from transparent/top level through to the bottom
fu! <SID>HiLinkTrace()
  " get highlighting linkages
  redir! @a
  silent! hi
  redir END

  " initialize with top-level highlighting
  let firstlink = synIDattr(synID(line("."),col("."),1),"name")
  let lastlink  = synIDattr(synIDtrans(synID(line("."),col("."),1)),"name")
  let translink = synIDattr(synID(line("."),col("."),0),"name")
  let nxtlink   = substitute(@a,'.*'.firstlink.' xxx links to \(\a\+\).*','\1','')

  " if transparent link isn't the same as the top highlighting link,
  " then indicate it with a leading "T:"
  if firstlink != translink
	let hilink="T:".translink." -> ".firstlink
  else
	let hilink=firstlink
  endif

  " trace through the linkages
  if firstlink != lastlink
	let no_overflow= 0
	while nxtlink != lastlink && no_overflow < 10
	  let hilink      = hilink ." -> ". nxtlink
	  let nxtlink     = substitute(@a,'.*'.nxtlink.' xxx links to \(\a\+\).*','\1','')
	  let no_overflow = no_overflow + 1
	endwhile

	" put last nxtlink in (which should be the same as lastlink)
	let hilink = hilink ." -> ". lastlink
  endif
  echo hilink
endfunction

" ---------------------------------------------------------------------
