"
" spell.vim - VIM macros for ispell "integration"
" Copyright (c) 2001 Marius Gedminas <mgedmin@delfi.lt>
"
" Required files:
"   ~/.vim/vimspell.sh
"   ~/.vim/ispell-filter.sh
"
" Bugs
"   ispell-filter does not work in GUI
"   these `normal <CR><C-L>' do not seem to work
"
" $Id: spell.vim 51 2005-01-23 20:44:24Z mg $
"

" Functions
"   Do_Spell_Hl()	highlights misspelled words
"   Do_Spell_Clear()	clears SpellErrors highlight group
"   Do_Spell_Line()	filters current line through ispell
"   Do_Spell_Visual()	filters selected area through ispell
"   Do_Spell_Add()	adds <cword> to .ispell_language
"
"   Spell_Foo()		set the following variables for language Foo
"
" Variables
"   spell_dict		string containing "-d language" to be passed to ispell
"   spell_file		local dictionary file (".ispell_language")

" Functions								{{{1

function! Do_Spell_Hl()
  exec "source `$HOME/.vim/vimspell.sh" g:spell_dict "< %`"
  normal <CR><CR>
endf

function! Do_Spell_Clear()
  syntax clear SpellErrors
  normal <CR>
endf

function! Do_Spell_Line()
  exec ".!$HOME/.vim/ispell-filter.sh" g:spell_dict
  normal <CR><C-L>
endf

function! Do_Spell_Visual()
  exec "'<,'>!$HOME/.vim/ispell-filter.sh" g:spell_dict
  normal <CR><C-L>
endf

function! Do_Spell_Add()
  exec "!echo <cword> >> " g:spell_file
  normal <CR><CR>
endf

" English								{{{1
function! Spell_En()
  let g:spell_dict = "-d british"
  let g:spell_file = ".ispell_english"
endf
command! SpellEn	call Spell_En()

" Lithuanian								{{{1
function! Spell_Lt()
  let g:spell_dict = "-d lietuviu"
  let g:spell_file = ".ispell_liet"
endf
command! SpellLt	call Spell_Lt()

" Default								{{{1

if !exists("spell_dict")
  call Spell_En()
endif
